# -*- coding: utf-8 -*-
"""
Módulo que contiene clases y un programa para trabajar con las medidas
tomadas por un mareógrafo, que contienen la altura de las olas,
el nivel del mar y la hora de la medida

@author: TO-DO
@date:   may 2019
"""

from datetime import datetime
from datetime import timedelta
from typing import List
import locale
import matplotlib.pyplot as plt

class IndiceIncorrecto(Exception):
    """
    Indica que un índice de la lista de medidas es incorrecto,
    por estar fuera del intervalo [-num,num), siendo num el número
    de medidas en la lista
    """

class NoHaySuficientesMedidas(Exception):
    """
    No hay suficientes medidas en la lista para satisfacer la petición
    """

class Medida:
    """
    Medida tomada por un mareógrafo, que contiene la altura de las
    olas, el nivel del mar y la hora de la medida.

    Attributes:
        __hora_gmt: es un texto con la fecha y hora referidos
                  al meridiano de Greenwich.
        __nivel_del_mar: en metros
        __altura_olas: en metros
    """

    def __init__(self, hora_gmt: str, nivel_del_mar: float,
                 altura_olas: float):
        """
        Constructor al que se le pasan los datos de la medida
        Args:
            nivel_del_mar nivel del mar en metros
            altura_olas  altura de las olas en metros
        """
        self.__nivel_del_mar: float = nivel_del_mar
        self.__altura_olas: float = altura_olas
        self.__hora_gmt: str = hora_gmt

    def get_hora_gmt(self)-> str:
        """
        Observador de la hora de la medida
        Returns:
            la hora GMT en forma de un texto
        """
        return self.__hora_gmt

    def get_nivel_del_mar(self)-> float:
        """
        Observador del nivel del mar
        Returns:
            el nivel del mar en metros
        """
        return self.__nivel_del_mar

    def get_altura_olas(self)-> float:
        """
        Observador de la altura de las olas
        Returns:
            la altura de las olas en metros
        """
        return self.__altura_olas

class Mareografo:
    """
    Contiene Software de análisis de los datos obtenidos de un
    mareógrafo, que es un sistema que mide la altura del nivel del mar
    en un punto concreto

    Attributes:
        __lista: guarda una lista de medidas obtenidas en instantes
                 consecutivos a lo largo del tiempo. El primer elemento
                 de la lista es el más antiguo, y el último el
                 más nuevo.
        __codigo: un String que identifica el mareógrafo.
        __cadencia: el tiempo entre cada medida, en minutos.
    """

    def __init__(self, codigo: str, cadencia: int):
        """
        Constructor: copia en los respectivos atributos el código
        y la cadencia. Adeáas, crea el atributo lista vacío.
        Args:
            codigo     el código del mareografo
            cadencia   el intervalo entre las medidas, en minutos
        """
        # To-Do

    def num_medidas(self)-> int:
        """
        Retorna el numero de medidas que contiene la lista
        """
        return len(self.__lista)


    def get_medida(self, indice: int)-> Medida:
        """
        Retorna la medida que ocupa la posicion indice en la lista
        (se numeran desde cero y admiten valores negativos).
        Si indice indica una medida no
        existente en la lista se lanzará IndiceIncorrecto

        Args:
            indice: el índice de la medida solicitada
        Returns:
            La medida solicitada
        Raises:
            IndiceIncorrecto: si indice está fuera del intervalo
                              [-num,num), siendo num el número de
                              medidas en la lista
        """
        if not -len(self.__lista) <= indice < len(self.__lista):
            raise IndiceIncorrecto
        else:
            return self.__lista[indice]


    def altura_media_olas(self, intervalo: int)-> float:
        """
        Retorna la altura media de las olas en el último intervalo
        expresado por el parámetro intervalo. Si el
        tamaño de la lista es inferior
        a intervalo//cadencia lanza NoHaySuficientesMedidas.
        Args:
            intervalo    el intervalo para calcular la media,
                         en minutos
        Returns:
            la altura media de las olas, en metros
        Raises:
            NoHaySuficientesMedidas si el tamaño de la lista es
            inferior a intervalo//cadencia
        """
        # To-Do

    def hay_altura_mayor(self, alt: float)-> bool:
        """
        Retorna un booleano que indica si entre las medidas
        hay alguna cuya suma del nivel del mar y de altura de olas
        supere el parámetro alt.
        Args:
            alt   altura a comprobar, en metros
        Returns:
            booleano que indica si se ha superado o no la altura alt
        """
        # To-Do

    def listado(self):
        """
        Muestra en la pantalla un
        listado de todas las medidas del mareógrafo.
        El listado se hace en columnas e incluye una cabecera.
        """
        # To-Do

    def conecta(self):
        """
        Se conecta por satélite con el mareógrafo y obtiene sus
        medidas, guardándolas en la lista. Se da hecha una
        simulación de este método que toma las medidas de un fichero
        de texto llamado mareografo.txt
        """
        # Partimos de una fecha ficticia en el pasado
        fecha = datetime(2019, 3, 20, 12, 30, 59, 0)
        cadencia_td = timedelta(minutes=self.__cadencia)
        locale.setlocale(locale.LC_ALL, "es_ES.UTF-8")
        nombre_fichero = "mareografo.txt"
        try:
            with open(nombre_fichero, "r") as fich:
                # lee el fichero línea a línea
                for linea in fich:
                    # Partir la línea en partes
                    numeros = linea.split()
                    if len(numeros) != 2:
                        raise IOError \
                            ("No hay dos números en la línea")
                    # Pasar a string la fecha y hora
                    fecha_texto: str = fecha.isoformat(timespec='seconds')
                    # crea una medida con los datos y la mete en la lista
                    self.__lista.append(
                        Medida(fecha_texto, locale.atof(numeros[0]),
                               locale.atof(numeros[1])))
                    # Sumar la cadencia a la fecha y hora
                    fecha = fecha + cadencia_td

        except IOError as err:
            print("Error abriendo fichero: "+nombre_fichero)
            print(err)


    def muestra_grafica(self):
        """
        Parte avanzada: Muestra una gráfica del nivel del mar,
        la altura de las olas, y su suma
        """
        # To-Do (Parte avanzada)

def main():
    """
    Programa principal, que crea un objeto de la clase Mareografo
    para Puerto Chico, muestra en pantalla los datos de la primera
    medida de la lista y de la última, muestra en pantalla
    la altura media de las olas en los últimos 60 minutos, y
    muestra en pantalla si hubo olas mayores a 10 o a 4 metros
    """
    # To-Do

if __name__ == "__main__":
    main()
